==> apracjs-13.1.29.tgz: package/lib/handlers/static.js <==
﻿/*!
 * apracjs
 * Copyright(c) 2017 by wssf
 * 2017-04-26
 * 静态文件的请求，支持压缩和缓存，同时处理R.axd里面的资源请求
 */

'use strict'
var Handler = require('./handler.js');
var utils = require('../common/utils.js');
var config = require('../config.js').apracConfig;
var debug = require('debug')('aprac:staticHandler');
var delegate = require('../common/utils.js').createDelegate;
var iconv = require('iconv-lite');
var path = require('path');
var fs = require('fs');


class StaticHandler extends Handler {
  constructor(app, distributor) {
    super(app, '*', distributor);

    this.resourceMap = {
      "js": new JavaScriptResouce('application/x-javascript; charset=utf-8'),
      "css": new StyleSheetResouce('text/css; charset=' + config.defaultEncoding)
    };
  }

  processRequest(req, res, done) {
    super.processRequest(req, res, done);

    var parsedUrl = req.parsed;
    var ext = parsedUrl.extname.slice(1); // .js->js
    if (!ext.match(config.fileMatch)) {
      debug(`apracjs不支持此类请求, url:${parsedUrl.filename} 发起来源:${utils.getIP(req)}`);
      done();
      return;
    }

    var pathname = parsedUrl.pathname;
    var filepath = parsedUrl.filename;
    fs.stat(filepath, delegate(this, function(err, stat) {
      if (err) {
        debug('文件不存在:%s', filepath);
        res.statusCode = 404;
        done();
        return;
      }

      var lastModified = stat.mtime;
      if (utils.isRequestNotModified(req, lastModified)) {
        res.end();
        return;
      }

      fs.readFile(filepath, delegate(this, function(err, file) {
        if (err) {
          debug('文件读取错误:%s', filepath);
          res.statusCode = 500;
          done();
          return;
        }

        var contentType = config.mimeTypes[ext] || "text/plain";
        utils.setCacheLastModified(res, lastModified, config.env); // 产品发布环境缓存

        var resourceType = this.resourceMap[ext];
        if (resourceType) {
          var sourceItem = resourceType.process(req, file);
          file = sourceItem.file;
          contentType = sourceItem.contentType;
        }

        res.writeHead(200, {
          'Content-Type': contentType,
          '_Content-Length': file.length
        });
        res.write(file);
        res.end();
      }));
    }));
  }
}
module.exports = StaticHandler;

class ResourceType {
  constructor(contentType) {
    this.contentType = contentType;
  }

  process(req, file) {

  }
}

class StyleSheetResouce extends ResourceType {
  constructor(contentType) {
    super(contentType);
  }

  process(req, file) {
    var css = utils.getUTF8String(file);

    var baseUrl = req.parsed.path; // 如 /Carpa.WebExample/R.axd?Carpa.Web/skins/liquidSky/skin.css
    var basePath = "";
    var pos = baseUrl.lastIndexOf('/');
    if (pos >= 0) {
      basePath = baseUrl.substr(0, pos + 1);
    }
    const cUrl = "url(\"";
    css = css.replace(/url\(\"/g, cUrl + basePath); // url("menu/menuItem.gif"); 全部替换为 url("${basePath}menu/menuItem.gif");

    return { file: css, contentType: this.contentType };
  }
}

class JavaScriptResouce extends ResourceType {
  constructor(contentType) {
    super(contentType);
  }

  process(req, file) {
    var content = "//@ sourceURL=" + req.parsed.pathname + "\r\n"; // url值如 V5UITests/tests/dataBind1.js,方便浏览器调试
    content += utils.getUTF8String(file);
    content += "\r\nif (typeof(Sys) !== 'undefined') Sys.Application.notifyScriptLoaded();";
    return { file: content, contentType: this.contentType };
  }
}

==> apracjs-18.1.2.tgz: package/lib/handlers/static.js <==
﻿/*!
 * apracjs
 * Copyright(c) 2017 by wssf
 * 2017-04-26
 * 静态文件的请求，支持压缩和缓存，同时处理R.axd里面的资源请求
 */

'use strict'
var Handler = require('./handler.js');
var utils = require('../common/utils.js');
var config = require('../config.js').apracConfig;
var debug = require('debug')('aprac:staticHandler');
var delegate = require('../common/utils.js').createDelegate;
var fs = require('fs');


class StaticHandler extends Handler {
  constructor(app, distributor) {
    super(app, '*', distributor);

    this.resourceMap = {
      "js": new JavaScriptResouce('application/x-javascript; charset=utf-8'),
      "css": new StyleSheetResouce('text/css; charset=' + config.defaultEncoding)
    };
  }

  processRequest(req, res, done) {
    super.processRequest(req, res, done);

    var parsedUrl = req.parsed;
    var ext = parsedUrl.extname.slice(1); // .js->js
    if (!ext.match(config.fileMatch)) {
      debug(`apracjs不支持此类请求, url:${parsedUrl.filename} 发起来源:${utils.getIP(req)}`);
      done();
      return;
    }

    var filepath = parsedUrl.filename;
    fs.stat(filepath, delegate(this, function(err, stat) {
      if (err) {
        debug('文件不存在:%s', filepath);
        res.statusCode = 404;
        done();
        return;
      }

      var lastModified = stat.mtime;
      if (utils.isRequestNotModified(req, lastModified)) {// 检查请求头，检查静态文件是否过期或者最后更新时间是否没变，如果没变则不下载文件
        res.end(); // 如果文件没有修改，不需要输出文件给前端，直接完成，浏览器会使用缓存
        return;
      }

      fs.readFile(filepath, delegate(this, function(err, file) {
        if (err) {
          debug('文件读取错误:%s', filepath);
          res.statusCode = 500;
          done();
          return;
        }

        var contentType = config.mimeTypes[ext] || "text/plain";
        utils.setCacheLastModified(res, lastModified, config.env); // 设置缓存请求头 产品发布环境才缓存，开发不缓存

        var resourceType = this.resourceMap[ext];
        if (resourceType) {
          var sourceItem = resourceType.process(req, file);
          file = sourceItem.file;
          contentType = sourceItem.contentType;
        }

        res.writeHead(200, {
          'Content-Type': contentType,
          '_Content-Length': file.length
        });
        res.write(file); // 输出文件流到前端
        res.end();
      }));
    }));
  }
}
module.exports = StaticHandler;

class ResourceType {
  constructor(contentType) {
    this.contentType = contentType;
  }

  process(req, file) {

  }
}

class StyleSheetResouce extends ResourceType {
  constructor(contentType) {
    super(contentType);
  }

  process(req, file) {
    var css = utils.getUTF8String(file);

    var baseUrl = req.parsed.path; // 如 /Craba.WebExample/R.axd?Craba.Web/skins/liquidSky/skin.css
    var basePath = "";
    var pos = baseUrl.lastIndexOf('/');
    if (pos >= 0) {
      basePath = baseUrl.substr(0, pos + 1);
    }
    const cUrl = "url(\"";
    css = css.replace(/url\(\"/g, cUrl + basePath); // url("menu/menuItem.gif"); 全部替换为 url("${basePath}menu/menuItem.gif");

    return { file: css, contentType: this.contentType };
  }
}

class JavaScriptResouce extends ResourceType {
  constructor(contentType) {
    super(contentType);
  }

  process(req, file) {
    var content = "//@ sourceURL=" + req.parsed.pathname + "\r\n"; // url值如 V5UITests/tests/dataBind1.js,方便浏览器调试
    content += utils.getUTF8String(file);
    content += "\r\nif (typeof($app) !== 'undefined') $app.notifyScriptLoaded();";
    return { file: content, contentType: this.contentType };
  }
}
